;;; mdev.el --- Code to support multiple displays in XEmacs

;; Authors: Rick Farnbach <fsf@wv.mentorg.com> 
;;          Paul Furnanz <paulf@wv.mentorg.com>
;; Version: 1.1
;; Keywords: XEmacs, tty, X, displays

;; Copyright (C) 1995
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 1, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;; BUG REPORTS: Send mail to the author(s) above

;;; INSTALLATION:
;;;
;;;   o  This requires XEmacs
;;;   o  Put this file somewhere in your load-path and byte-compile it.
;;;   o  You need gnuserv up and running
;;;   o  Put this in your ~/.emacs:
;;;               (require 'mdev)
;;;   o  If you want to be able to type "C-xC-c" to kill a device or
;;;      to kill this session of XEmacs (if last device), then put the
;;;      following in your ~/.emacs after the require line:
;;;               (global-set-key "\C-x\C-c" 'mdev-save-buffers-kill-emacs)
;;;

;;; USAGE:
;;;
;;;   o  To create a device run the mdev command from a shell.  See
;;;      documentation in the mdev script for command line options.
;;;   o  Use "M-x mdev-delete" to delete the current device.  If you
;;;      set up the optional key binding, then just type "C-xC-c"
;;;

;; LCD Archive Entry:
;; mdev|Paul Furnanz & Rick Farnback|paulf@wv.mentorg.com & fsf@wv.mentorg.com|
;; Manage multiple devives (TTY and X) with one XEmacs session|
;; 11/06/95|Version 1.1||


(defvar mdev-pid-alist nil 
  "List of attached tty's.  This is not a user setable variable")

(defun mdev-new-tty (tty pid)
  "Internal function used to attach to a tty device"
  (let ((dev (make-tty-device tty)))
    (make-frame nil dev)
    (push (cons dev pid) mdev-pid-alist)))

(defun mdev-new-x (display)
  "Internal function to attach to an X display"
  (make-frame nil (make-x-device display)))

(defun mdev-delete (&optional dev kill)
  "Detatch the device DEV from this XEmacs.  If DEV is not given,
  detach current device.  If KILL is t, then really kill the last device."
  (interactive)
  (if (and (eq (length (device-list)) 1)
	   (not kill))
      (error "Tried to delete last device!"))
  (or dev (setq dev (selected-device)))
  (let ((mdev-pid (assq dev mdev-pid-alist)))
    (if mdev-pid
	(progn
	  (delete mdev-pid mdev-pid-alist)
	  (signal-process (cdr mdev-pid) 1))))
  (delete-device dev)
  (select-device (car (device-list))))

(defun mdev-save-buffers-kill-emacs nil
  "Kill current device or kill emacs if this is the last device"
  (interactive)
  (mdev-delete nil t))


(provide 'mdev)
